// *****************************************************************************
//
//	File		:  stiquitoMSP430.c
//
//	Author		:	Andrew McClain
//
//	Description:	Includes main and supporting functions for Stiquito
//					basic operation
//
//	Features/Information:
//			-Texas Instruments MSP430F1122 MicroProcessor
//			-Supports dynamic switching between 1-dof and 2-dof modes
//				based on jumper setting
//			-Supports dynamic speed change based on on-board potentiometer
//				(through 10-bit A/D converter).  Complete leg cycle ranges
//				from 3-11secs
//
//	Terminology	:	Cycle - One complete operation of all legs.  Once each leg
//						has been pulled back and (in 2-dof, pulled forward)
//						returned to inital state, one cycle is complete.
//					Stage - One 16th of a cycle.
//					Step - One 16th of a Stage.
//
//	Revisions
//------------------------------------------------------------------------------
//
//	Version 1.0
//	Developers	:	Andrew McClain
//	Date		:	10 Nov 2004
//	Purpose	:	Create initial code for Stiquito that satisfies requirements
//					stated in Stiquito book.
//
//------------------------------------------------------------------------------
//
//	Version 1.1
//	Developers	:	Andrew McClain
//	Date		:	22 Nov 2004
//	Purpose	:	Tidy code and comment for readability/future expansion
//
//------------------------------------------------------------------------------
//
//	Special Notes:  This file (stiquitoMSP430v1.1 1d-ns.c) has
//						one dof only, no speed control
//
// *****************************************************************************

#include <msp430x11x2.h>

// *************************
// *  Function Prototypes  *
// *************************
//
// See function descriptions at function code headers

void toggle(int);
int power(int);
void one_degree(void);

// **********************
// *  Global Variables  *
// **********************

int degree =1;		// Determines degree-of-freedom.  Start with 2 dof, but
					// jumper is checked before legs are moved.

int stage = 0;  	// Tracks the stage (0-15) of cycle.

int step = 0;		// Tracks the steps (0-15) within the stage.

int leg[4] = {0,1,0,0};	// Array to track which leg is on (used by toggle() )

int speed_reg = 1563; 		// Value used to continuously increment speed register,
					// set to 5 sec cycle time

int current_speed = -1; // fastest speed setting (1-32)

// **********
// *  Main  *
// **********

void main (void) {

	BCSCTL1 &= ~(RSEL0 + RSEL1 + RSEL2);	//Sets RSELx bits to 000, slowest speed
	DCOCTL  &= ~(DCO0 + DCO1 + DCO2);	// Sets DCOx bits to 000, slowest speed

	WDTCTL = WDT_MDLY_32;	// WDT 1s interval timer.  This statment
							// is required even though WDT is not implemented

	CCTL0 = CCIE;	// CCR0 interrupt enabled
	TACTL = TASSEL_2 + MC_2;	// SMCLK, contmode.  See TI sample code for description

	// P1 is set as follows:
	// P1.1:  Tripod A 2-dof - output
	// P1.0:  Tripod B 2-dof - output
	// P1.2:  Tripod B - output
	// P1.3:  Tripod A - output
	// P1.4 thru P1.7 : JTAG -Input

	P1DIR = 0x0F;	// Init P1.x
	P1OUT = 0x00;	// Turn off legs and other outputs

	// P2 is set as follows:
	// P2.0: ADC input from POT to adjust speed
	// P2.2 and P2.0 : routed to Protype Area
	// P2.3:  Tripod B LED - output
	// P2.4:  Tripod A LED - output
	// P2.5:  2-dof jumper - input

	P2DIR = 0x1E;	// Init P2.x
	P2OUT = 0x00;	//turn off legs and other outputs
	P2OUT |= 0x18;	// turn off leds (inverse logic)

	CCR0 = speed_reg;	// Set initial timer count register

	__enable_interrupt();	// Enable global interrupts (Start Timer)

	_BIS_SR(LPM0_bits + GIE);	// Enter LPM0 w/ interrupt (Low power standby)

}


// ***************
// *  Functions  *
// ***************


// *****************************************************************************
//
//	Name		:	watchdog_timer
//
//	Description	:	WDT interrupt service routine.  Not currently implemented
//
// *****************************************************************************

#pragma vector=WDT_VECTOR
__interrupt void watchdog_timer (void) {

}	// end void watchdog_timer(void)


// *****************************************************************************
//
//	Name        :	Timer_A3
//
//	Description :	Timer A0 interrupt service routine
//					This function occurs whenever CCR0 register reaches 0.  (This
//					register is then incremented inside the isr)  Executes
//					movement based on global dof flag.  Increments step and stage
//					(if necessary).
//
// ****************************************************************************

#pragma vector=TIMERA0_VECTOR
__interrupt void Timer_A3 (void)
{
	CCR0 += speed_reg;	// Reincrement timer

	one_degree();

	step++;	// Increment step number

	// if step limit has been reached (end of stage),
	//    reset step count and increment stage.
	if (step == 16) {
		step = 0;
		stage++;
		if (stage == 16) {	// if cycle is complete, reset stage #
			stage = 0;
		}

	}	// end of step/stage increment

} // end of timer_isr

// *****************************************************************************
//
//	Name        :	toggle
//
//	Description :	This function toggles the leg output corresponding to the
//					number passed to it.  This is achieved by checking current
//					leg status and creating/using a mask on p1out.
//
// ****************************************************************************


void toggle (int leg_num) {

	int mask;

	mask = power(leg_num);	// Create mask for leg_num

	if (leg[leg_num] == 1) {	// If leg is on, turn off

		mask = 0xFF - mask;	// Turn off leg by ANDing with FF-mask
		P1OUT &= mask;
		leg[leg_num] = 0;	// Set leg status flag to 0

		switch (leg_num) {	// If leg being toggled is a main leg,
							// toggle matching LED
		    case 3:
		        P2OUT |= 0x10;   // DEactivate Tripod A LED (inverse logic)
		        break;
		    case 2:
		        P2OUT |= 0x08;   // DEactivate Tripod B LED (inverse logic)
		        break;
		    default:
		        break;
		}	// end of led switch

	} else {	// If leg is off, turn on

		P1OUT |= mask;	// Turn on leg by ORing with mask

		leg[leg_num] = 1;	// Set leg status flag to 1

		switch (leg_num) {	// If leg being toggled is a main leg,
							// toggle matching LED
		    case 3:
		        P2OUT &= 0xEF;   // activate Tripod A LED
		        break;
		    case 2:
		        P2OUT &= 0xF7;   // activate Tripod B LED
		        break;
		    default:
		        break;

		}

	} // end of leg on/off if/else check

} // end of toggle function

// *****************************************************************************
//
//	Name        :	power
//
//	Description :	This function is used to create mask by raising 2 to a power
//
// ****************************************************************************

int power(int power) {
	int i;
	int r=1;

	for(i=0; i<power; i++)
		r=r*2;

	return r;

 }


// *****************************************************************************
//
//	Name        :	one_degree
//
//	Description :	This function contains the switch statment for 1-dof operation.
//					Legs are toggled based on stage number.
//
// ****************************************************************************

void one_degree() {

	switch (stage) {

		case 0:	// a1 (leg 3) is in quick step
				toggle(3);
				break;

		case 1:
		case 2:
		case 3:
		case 4:	// a1 (leg 3) are in slow step
				if (step%2 == 0) {
					toggle(3);
				}
				break;

		case 5:
		case 6:
		case 7:	// Nothing is happening in these stages
				break;

		case 8:	// b1 (leg 2) is in quick step
				toggle(2);
				break;

		case 9:
		case 10:
		case 11:
		case 12:	// b1 legs (2) are in slow step
				if (step%2 == 0) {
					toggle(2);
				}
				break;

		case 13:
		case 14:
		case 15:	// Nothing is happening in these stages
				break;

		default:
				break;

	}	// end of stage switch

}	// end of 2-degree switch function




